# -*- coding: UTF-8 -*-
import hashlib
import random
import requests
import time
import re
import sys
import os
from requests.exceptions import RequestException
from bs4 import BeautifulSoup

PY3 = sys.version_info > (3, )
unicode = str if PY3 else unicode

s = requests.Session()
m = hashlib.md5()

class mkYouDaoTranslator(object):
    def __init__(self):
        self.headers2 = {
            'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:57.0) Gecko/20100101 Firefox/57.0',
            'Referer': 'http://fanyi.youdao.com/',
            'contentType': 'application/x-www-form-urlencoded; charset=UTF-8'
        }
        self.headers1 = {
            'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10.14; rv:63.0) Gecko/20100101 Firefox/63.0',
            'Origin': 'https://ai.youdao.com',
            'Accept-Language': 'zh-CN,zh;q=0.8,zh-TW;q=0.7,zh-HK;q=0.5,en-US;q=0.3,en;q=0.2',
            'Referer': 'https://ai.youdao.com/product-fanyi.s',
            'contentType': 'application/x-www-form-urlencoded; charset=UTF-8',
        }
        self.url2 = 'http://fanyi.youdao.com/translate_o?smartresult=dict&smartresult=rule'
        self.url1 = 'https://aidemo.youdao.com/trans'

    def translate(self, text, dest, src):
        time.sleep(0.1)
        return self.trans1(text,dest,src)

    def trans1(self, text, dest, src):
        s.get('https://aidemo.youdao.com/trans')
        data1 = {
            'q': text,
            'from': 'Auto',
            'to': dest
        }
        resp = s.post(self.url1, headers=self.headers1, data=data1)
        try:
            if(resp.json()['translation']):
                return resp.json()['translation'][0]
        except:
            print('使用有道备用翻译通道2')
            return self.trans2(text,dest,src)

    def trans2(self, text, dest, src):
        s.get('http://fanyi.youdao.com/')
        salf = str(int(time.time() * 1000) + random.randint(0, 9))
        n = 'fanyideskweb' + text + salf + "rY0D^0'nM0}g5Mm1z%1G4"
        m.update(n.encode('utf-8'))
        sign = m.hexdigest()
        data2 = {
            'i': text,
            'from': 'AUTO',
            'to': dest,
            'smartresult': 'dict',
            'client': 'fanyideskweb',
            'salt': salf,
            'sign': sign,
            'doctype': 'json',
            'version': "2.1",
            'keyfrom': "fanyi.web",
            'action': "FY_BY_DEFAULT",
            'typoResult': 'false'
        }
        resp = s.post(self.url2, headers=self.headers2, data=data2)
        try:
            return resp.json()['translateResult'][0][0]['tgt']
        except:
            return self.trans3(text,dest,src)

    def trans3(self, text, dest, src):
        print('使用接口，仅支持中英互译,将翻译为英文')
        params = {
            'keyfrom': 'woodcol',
            'key': '1522180019',
            'type': 'data',
            'doctype': 'json',
            'version': '1.1',
            'q': text
        }
        try:
            r = requests.get('http://fanyi.youdao.com/openapi.do', params=params)
            r.raise_for_status()
            self.result = r.json()
            return self.result['translation'][0]
        except RequestException as e:
            print('有道免费接口遭遇封锁，请联系作者:https://github.com/mythkiven/mkTranslate')
            sys.exit()



# -*- coding: utf-8 -*-

import requests
import random
from mkTranslation import utils,network
from mkTranslation.network import TimeoutAdapter,TokenAcquirer
from mkTranslation.utils import PY3
from mkTranslation.constants import DEFAULT_USER_AGENT, LANGCODES, LANGUAGES, SPECIAL_CASES
from mkTranslation.model import Translated, Detected

EXCLUDES = ('en', 'ca', 'fr')

class mkGoogleTranslator(object):
    def __init__(self, service_urls=None, user_agent=DEFAULT_USER_AGENT,proxies=None, timeout=None):

        self.session = requests.Session()
        if proxies is not None:
            self.session.proxies = proxies
        self.session.headers.update({
            'User-Agent': user_agent,
        })
        if timeout is not None:
            self.session.mount('https://', TimeoutAdapter(timeout))
            self.session.mount('http://', TimeoutAdapter(timeout))

        self.service_urls = service_urls or ['translate.google.com']
        self.token_acquirer = TokenAcquirer(session=self.session, host=self.service_urls[0])

        try:
            from hyper.contrib import HTTP20Adapter
            self.session.mount(network.BASE, HTTP20Adapter())
        except ImportError:
            pass

    def translate(self, text, dest='en', src='auto'):
        dest = dest.lower().split('_', 1)[0]
        src = src.lower().split('_', 1)[0]

        if src != 'auto' and src not in LANGUAGES:
            if src in SPECIAL_CASES:
                src = SPECIAL_CASES[src]
            elif src in LANGCODES:
                src = LANGCODES[src]
            else:
                raise ValueError('invalid source language')

        if dest not in LANGUAGES:
            if dest in SPECIAL_CASES:
                dest = SPECIAL_CASES[dest]
            elif dest in LANGCODES:
                dest = LANGCODES[dest]
            else:
                raise ValueError('invalid destination language')

        if isinstance(text, list):
            result = []
            for item in text:
                translated = self.translate(item, dest=dest, src=src)
                result.append(translated)
            return result

        origin = text
        data = self._translate(text, dest, src)

        translated = ''.join([d[0] if d[0] else '' for d in data[0]])
        extra_data = self._parse_extra_data(data)
        try:
            src = data[2]
        except Exception:
            pass

        pron = origin
        try:
            pron = data[0][1][-2]
        except Exception:
            pass
        if not PY3 and isinstance(pron, unicode) and isinstance(origin, str):
            origin = origin.decode('utf-8')
        if dest in EXCLUDES and pron == origin:
            pron = translated
        if not PY3:
            if isinstance(src, str):
                src = src.decode('utf-8')
            if isinstance(dest, str):
                dest = dest.decode('utf-8')
            if isinstance(translated, str):
                translated = translated.decode('utf-8')
        result = Translated(src=src, dest=dest, origin=origin,text=translated, pronunciation=pron, extra_data=extra_data)
        return result

    def _pick_service_url(self):
        if len(self.service_urls) == 1:
            return self.service_urls[0]
        return random.choice(self.service_urls)

    def _translate(self, text, dest, src):
        if not PY3 and isinstance(text, str):
            text = text.decode('utf-8')

        token = self.token_acquirer.do(text)
        params = utils.build_params(query=text, src=src, dest=dest,token=token)
        url = network.TRANSLATE.format(host=self._pick_service_url())
        r = self.session.get(url, params=params)
        data = 'cannot tx !!!'
        if(r.text):
            data = utils.format_json(r.text)
        return data
    def translate_text(self,text,dest):
        if isinstance(text, list):
            result = []
            for item in text:
                lang = self.detect(item)
                result.append(lang)
            return result
        origin = text
        data = self._translate(text, dest, src='auto')
        translated = ''.join([d[0] if d[0] else '' for d in data[0]])
        extra_data = self._parse_extra_data(data)
        try:
            src = data[2]
        except Exception:
            pass

        pron = origin
        try:
            pron = data[0][1][-2]
        except Exception:
            pass
        if not PY3 and isinstance(pron, unicode) and isinstance(origin, str):
            origin = origin.decode('utf-8')
        if dest in EXCLUDES and pron == origin:
            pron = translated
        if not PY3:
            if isinstance(src, str):
                src = src.decode('utf-8')
            if isinstance(dest, str):
                dest = dest.decode('utf-8')
            if isinstance(translated, str):
                translated = translated.decode('utf-8')
        result = Translated(src=src, dest=dest, origin=origin,text=translated, pronunciation=pron, extra_data=extra_data)
        return result
    def detect(self, text):
        if isinstance(text, list):
            result = []
            for item in text:
                lang = self.detect(item)
                result.append(lang)
            return result

        data = self._translate(text, dest='en', src='auto')

        src = ''
        confidence = 0.0
        try:
            src = ''.join(data[8][0])
            confidence = data[8][-2][0]
        except Exception:
            pass
        result = Detected(lang=src, confidence=confidence)
        return result

    def _parse_extra_data(self, data):
        response_parts_name_mapping = {
            0: 'translation',
            1: 'all-translations',
            2: 'original-language',
            5: 'possible-translations',
            6: 'confidence',
            7: 'possible-mistakes',
            8: 'language',
            11: 'synonyms',
            12: 'definitions',
            13: 'examples',
            14: 'see-also',
        }
        extra = {}
        for index, category in response_parts_name_mapping.items():
            extra[category] = data[index] if (index < len(data) and data[index]) else None
        return extra


# -*- coding:utf-8 -*-
import os
import re
import sys
import json
import string
import requests
from mkTranslation.translate_google import mkGoogleTranslator
from mkTranslation.translate_youdao import mkYouDaoTranslator
sys.path.append("..")

class mkTranslator(object):

    def get_file(self,path):
        if(not os.path.exists(path)):
            pathArray = path.split('/')
            fileName = pathArray[len(pathArray)-1]
            return os.path.join(os.path.abspath('.'),fileName)
        return path

    def translate_i18ns(self,dest,word,language):
        if(not language):
            return 'null'
        tlanguage ='translations.'+language
        uri = "https://i18ns.com/translate/_search"
        headers = {"Content-Type":"application/json","Authorization":"Basic aTE4bnM6KioqKioq","user-agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.14; rv:63.0) Gecko/20100101 Firefox/63.0"}
        body = json.dumps({"query": {"simple_query_string":{"query":word,"fields":[tlanguage],"minimum_should_match": "100%","default_operator": "AND","analyzer": "smartcn"}}}) 
        response = requests.post(url=uri,data=body,headers=headers,timeout=120)
        if response.status_code != 200 :
            return 'null'
        txd = []
        try:
            tx=json.loads(response.text,encoding="UTF-8")
            txd=tx['hits']['hits'][0]['_source']['translations']
        except Exception as e:
            return 'null'
        if(len(txd)>1):
            isExist = False
            for x in txd:
                if(language == x['lang']):
                    orit = json.dumps(x[language],ensure_ascii=False)
                    orit = orit.replace('[','').replace(']','').replace('"','')
                    if(word == orit):
                        isExist = True
            if(isExist):
                for x in txd:
                    if(dest == x['lang']):
                        print('use \'https://i18ns.com/\' translation '+word+' to '+x[dest][0] + ',others use google translation')
                        return x[dest][0]+''
        return 'null'

    def translate(self,word,destination,language,channel):
        if(len(word)==0):
            return 'null'
        try:
            tx = self.translate_i18ns(destination,word,language)
            if(tx != 'null'):
                return tx
        except Exception as e:
            print('')
        if(channel == 'google'):
            return mkGoogleTranslator().translate(word, dest=destination).text
        else:
            return mkYouDaoTranslator().translate(word,destination,language)
        

    def fix_tx(self,txt):
        if(txt.find('% ld')!=-1 or txt.find('% @')!=-1):
            tsing = re.search(r'%\s*(@|ld)\s*/\s*%\s*(ld|@)',txt)
            if(tsing):
                tsing = tsing.group(0)
                txt = txt.replace(tsing,tsing.replace(' ',''))
        return txt

    def write_tx(self,oldfile,newfile,reg,creg,des,lan,channel):
        f = open(oldfile)
        line = f.readline()
        txd = ''
        while line:
            line = line.replace('\n','')
            if(len(line)==0):
                line = f.readline()
                continue
            originLine = line
            print('original:'+originLine)
            line = re.findall(reg,line)
            if(len(line) and line[0]):
                txc = self.translate(line[0],des,lan,channel)
                if(len(txc)):
                    originLine = re.sub(reg,creg.replace('content',txc), originLine)
                else:
                    print('translate fail:' + line)
            elif(reg==creg==r'text'):
                originLine = self.translate(originLine,des,lan,channel)
            else:
                print('skip: '+originLine)
            originLine = self.fix_tx(originLine)
            print('translated:'+originLine)
            txd += originLine + '\n'
            print('-----')
            line = f.readline()
        f = open(newfile,'w+')
        f.write(txd)
        f.close()

    def translate_text(self,text, destination,sourcelanguage,channel):
        channel = channel if channel else 'google'
        if(channel == 'google'):
            print(mkGoogleTranslator().translate_text(text, dest=destination).text)
        else:
            print(mkYouDaoTranslator().translate(text,destination,sourcelanguage))

    def translate_doc(self,filepath, destination,sourcelanguage,channel):
        channel = channel if channel else 'google'

        filepath = self.get_file(filepath)
        pathArray = filepath.split('/')
        oldFileName = pathArray[len(pathArray)-1]
        fileType = oldFileName.split('.')[len(oldFileName.split('.'))-1]
        currentPath =  filepath.replace('/' + oldFileName,'') if len(pathArray)>2 else  os.path.abspath('.')
        newFile = os.path.join(currentPath, 'translate_'+destination+'_by_'+channel+'_'+oldFileName)
        txd = ''
        print('translating..')

        # text
        if(fileType.lower() == 'text' or  fileType.lower() == 'txt'):
            self.write_tx(filepath,newFile,r"text",r"text",destination,sourcelanguage,channel)
            print('translation completed')

        # oc:xx.string
        elif(fileType.lower() == 'strings'):
            self.write_tx(filepath,newFile,r"=\s*\"(.+?)\"\s*;",'="'+'content'+'";',destination,sourcelanguage,channel)
            print('translation completed')
        # java:xx.xml
        elif(fileType.lower() == 'xml'):
            self.write_tx(filepath,newFile,r">\s*(.+?)\s*</string>",'>'+'content'+'</string>',destination,sourcelanguage,channel)
            print('translation completed')
        else:
            f = open(filepath)
            line = f.readline()
            while line:
                if(len(line)):
                    txd += self.translate(line,destination,sourcelanguage,channel)+'\n'
                line = f.readline()
            f.close()
            f = open(newFile,'w+')
            f.write(txd)
            f.close()
            print('translation completed')